'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
var logger_1 = require("../logger");
var accesscontrol_1 = require("accesscontrol");
var _ = require('lodash');
var CrudAccessControl = /** @class */ (function () {
    function CrudAccessControl(resource, grants) {
        this.resource = resource;
        this.grants = grants;
        this.ac = new accesscontrol_1.AccessControl(grants);
    }
    CrudAccessControl.prototype.check = function (user, op) {
        var _this = this;
        var _ac = this.ac;
        //check for `any` clearence: the user has accesss to any document in the target collection
        logger_1.default.trace("all roles of user: " + user.roles);
        var grant;
        var roles = this.filteredRoles(_ac, user.roles);
        logger_1.default.trace("filtered user roles: " + roles);
        var orgRoles = this.getOrgRoles(user);
        logger_1.default.trace("org roles: " + orgRoles);
        orgRoles = this.filteredRoles(_ac, orgRoles);
        logger_1.default.trace("filtered org roles: " + orgRoles);
        var assignedRoles = this.getAssignedRoles(user);
        logger_1.default.trace("assignedRoles roles: " + assignedRoles);
        assignedRoles = this.filteredRoles(_ac, assignedRoles);
        logger_1.default.trace("filtered assignedRoles roles: " + assignedRoles);
        logger_1.default.trace("check can " + roles + " " + op + "Any for " + this.resource);
        var anyRoles = roles.concat(orgRoles);
        var anyPermission = this.doCheck(_ac, anyRoles, op, 'Any', this.resource);
        if (anyPermission.granted) {
            grant = new Grant(anyPermission, 'any');
            grant.addVerifiedRoles(anyRoles);
        }
        else {
            //check for `own` clearence:
            var ownGrant = new Grant();
            ///check for `own` clearence for 'user'
            logger_1.default.trace("check can '" + roles + "' " + op + "Own for " + this.resource);
            var userOwnPermission = this.doCheck(_ac, roles, op, 'Own', this.resource);
            ownGrant.addVerifiedRoles(roles);
            //check for `organization:own` clearence
            if (userOwnPermission.granted) {
                ownGrant.addGrant(new Grant(userOwnPermission, 'own', 'user'));
            }
            //check for 'own' clearence for 'organization'
            logger_1.default.trace("check can '" + orgRoles + "' " + op + "Own for " + this.resource);
            var organizationOwnPermission = this.doCheck(_ac, orgRoles, op, 'Own', this.resource);
            if (organizationOwnPermission.granted) {
                ownGrant.addGrant(new Grant(organizationOwnPermission, 'own', 'organization'));
            }
            //check for 'own' clearence for 'assignedRoles'
            logger_1.default.trace("check can '" + assignedRoles + "' " + op + "Own for " + this.resource);
            assignedRoles
                .map(function (assignedRole) { return ({ assignedRole: assignedRole, permission: _this.doCheck(_ac, [assignedRole], op, 'Own', _this.resource) }); })
                .filter(function (rolePermission) { return rolePermission.permission.granted; })
                .map(function (rolePermission) { logger_1.default.trace(rolePermission); return rolePermission; })
                .map(function (_a) {
                var assignedRole = _a.assignedRole, permission = _a.permission;
                return new Grant(permission, 'own', 'assigned', assignedRole);
            })
                .map(function (grant) { logger_1.default.trace("grant " + JSON.stringify(grant)); return grant; })
                .forEach(function (grant) { return ownGrant.addGrant(grant); });
            logger_1.default.trace("ownGrant: '" + JSON.stringify(ownGrant) + "'");
            if (ownGrant.granted) {
                grant = ownGrant;
            }
            else {
                grant = new Grant(); /* granted false */
            }
        }
        return grant;
    };
    CrudAccessControl.prototype.doCheck = function (_ac, roles, op, type, resource) {
        var permission;
        try {
            permission = _ac.can(roles)[op + type](resource);
        }
        catch (err) {
            logger_1.default.warn(err.message);
            permission = { granted: false };
        }
        return permission;
    };
    ;
    // remove inexistent roles
    CrudAccessControl.prototype.filteredRoles = function (_ac, roles) {
        if (_.isEmpty(roles))
            return [];
        return roles.filter(function (role) { return _ac.hasRole(role); });
    };
    CrudAccessControl.prototype.getOrgRoles = function (user) {
        var orgRoles = [];
        orgRoles = orgRoles.concat(_.uniq(user.roles
            .filter(function (role) { return role.indexOf('$organization') == 0; })
            .map(function (role) { return role.replace(/:.*:/, ':'); })));
        return orgRoles;
    };
    CrudAccessControl.prototype.getAssignedRoles = function (user) {
        var assignedRoles = [];
        assignedRoles = assignedRoles.concat(_.uniq(user.roles
            .filter(function (role) { return role.indexOf('$assigned:') == 0; })
            .map(function (role) { return role.replace(/:.*:/, ':'); })));
        return assignedRoles;
    };
    return CrudAccessControl;
}());
exports.CrudAccessControl = CrudAccessControl;
var Grant = /** @class */ (function () {
    function Grant(permission, _type, _ownerType, assignedRole) {
        this.granted = false;
        this.type = 'none'; /* none, any, own */
        this.ownerTypes = []; /* empty, organziaiton, user, assigned */
        this.permissions = [];
        this.verifiedRoles = [];
        this.assignedRoles = [];
        if (_type) {
            this.type = _type;
        }
        this.addPermission(permission);
        this.addOwnerType(_ownerType);
        this.addAssignedRole(assignedRole);
    }
    Grant.prototype.addGrant = function (grant) {
        if (!grant) {
            return;
        }
        this.type = grant.type;
        this.addPermission(grant.permissions[0]);
        this.addOwnerType(grant.ownerTypes[0]);
        this.assignedRoles = this.assignedRoles.concat(grant.assignedRoles);
        return this;
    };
    Grant.prototype.addOwnerType = function (ownerType) {
        if (!ownerType) {
            return;
        }
        this.ownerTypes.push(ownerType);
    };
    Grant.prototype.addPermission = function (permission) {
        if (!permission || !permission.granted) {
            return;
        }
        this.permissions.push(permission);
        this.granted = this.granted || permission.granted;
    };
    Grant.prototype.addVerifiedRoles = function (roles) {
        if (!roles) {
            return;
        }
        this.verifiedRoles = this.verifiedRoles.concat(roles);
    };
    Grant.prototype.addAssignedRole = function (role) {
        if (!role) {
            return;
        }
        this.assignedRoles.push(role);
    };
    return Grant;
}());
exports.Grant = Grant;
